#ifndef __IRR_I_IMAGE_MANAGER_INCLUDED__
#define __IRR_I_IMAGE_MANAGER_INCLUDED__

#include "IImageManager.h"
#include "irrTypes.h"
#include "path.h"
#include "IReadFile.h"
#include "IImage.h"
#include "ITexture.h"
#include "IWriteFile.h"

namespace irr
{

namespace video
{
    class IImage;
    class IImageLoader;
    class IImageWriter;

class IImageManager
{

public:

        // TODO rename to addImageLoader
		//! Adds an external image loader to the engine.
		virtual void addExternalImageLoader(IImageLoader* loader) = 0;

		//! Adds an external image writer to the engine.
		virtual void addExternalImageWriter(IImageWriter* writer) = 0;

		//! Retrieve the number of image loaders
//		virtual u32 getImageLoaderCount() const = 0;

		//! Retrieve the given image loader
//		virtual IImageLoader* getImageLoader(u32 n) = 0;
//
//		//! Retrieve the number of image writers
//		virtual u32 getImageWriterCount() const = 0;

		//! Retrieve the given image writer
//		virtual IImageWriter* getImageWriter(u32 n) = 0;

		//! Creates a software image from a file.
		virtual IImage* createImageFromFile(const io::path& filename) = 0;

		//! Creates a software image from a file.
		virtual IImage* createImageFromFile(io::IReadFile* file) = 0;

		//! Creates a software image from a byte array.
		/** \param useForeignMemory: If true, the image will use the data pointer
		directly and own it from now on, which means it will also try to delete [] the
		data when the image will be destructed. If false, the memory will by copied. */
		virtual IImage* createImageFromData(ECOLOR_FORMAT format,
			const core::dimension2d<u32>& size, void *data,
			bool ownForeignMemory=true, bool deleteForeignMemory = true) = 0;

		//! Creates an empty software image.
		virtual IImage* createImage(ECOLOR_FORMAT format, const core::dimension2d<u32>& size) = 0;


		//! Creates a software image from another image.
		virtual IImage* createImage(ECOLOR_FORMAT format, IImage *imageToCopy) = 0;

		//! Creates a software image from part of another image.
		virtual IImage* createImage(IImage* imageToCopy,
				const core::position2d<s32>& pos,
				const core::dimension2d<u32>& size) = 0;

		//! Creates a software image from part of a texture.
		virtual IImage* createImage(ITexture* texture,
				const core::position2d<s32>& pos,
				const core::dimension2d<u32>& size) = 0;


        virtual bool writeImageToFile(IImage* image, const io::path& filename,u32 param = 0) = 0;
        virtual bool writeImageToFile(IImage* image, io::IWriteFile * file, u32 param = 0) = 0;

//    public:
//
//		core::array<video::IImageLoader*> SurfaceLoader;
//		core::array<video::IImageWriter*> SurfaceWriter;

};

}
}

#endif

