#ifndef __IRR_C_IMAGE_MANAGER_INCLUDED__
#define __IRR_C_IMAGE_MANAGER_INCLUDED__

#include "IImageManager.h"
#include "irrArray.h"
#include "path.h"
#include "os.h"


namespace irr
{


namespace io
{
//    struct path;
    class IReadFile;
}

namespace video
{

    class ITexture;
    class IImage;



class CImageManager : public IImageManager
{

public:
        CImageManager(io::IFileSystem* fs);

        virtual ~CImageManager();
		//! Adds an external image loader to the engine.
		virtual void addExternalImageLoader(IImageLoader* loader);

		//! Adds an external image writer to the engine.
		virtual void addExternalImageWriter(IImageWriter* writer);

//
//        virtual IImage* createImageFromFile(const io::path& filename);
//
//        virtual IImage* createImageFromFile(io::IReadFile* file);

		//! Retrieve the number of image loaders
		virtual u32 getImageLoaderCount() const;

		//! Retrieve the given image loader
		virtual IImageLoader* getImageLoader(u32 n);

		//! Retrieve the number of image writers
		virtual u32 getImageWriterCount() const;

		//! Retrieve the given image writer
		virtual IImageWriter* getImageWriter(u32 n);

		//! Creates a software image from a file.
		virtual IImage* createImageFromFile(const io::path& filename);

		//! Creates a software image from a file.
		virtual IImage* createImageFromFile(io::IReadFile* file);

		//! Creates a software image from a byte array.
		/** \param useForeignMemory: If true, the image will use the data pointer
		directly and own it from now on, which means it will also try to delete [] the
		data when the image will be destructed. If false, the memory will by copied. */
		virtual IImage* createImageFromData(ECOLOR_FORMAT format,
			const core::dimension2d<u32>& size, void *data,
			bool ownForeignMemory=true, bool deleteForeignMemory = true);

		//! Creates an empty software image.
		virtual IImage* createImage(ECOLOR_FORMAT format, const core::dimension2d<u32>& size);


		//! Creates a software image from another image.
		virtual IImage* createImage(ECOLOR_FORMAT format, IImage *imageToCopy);

		//! Creates a software image from part of another image.
		virtual IImage* createImage(IImage* imageToCopy,
				const core::position2d<s32>& pos,
				const core::dimension2d<u32>& size);

		//! Creates a software image from part of a texture.
		virtual IImage* createImage(ITexture* texture,
				const core::position2d<s32>& pos,
				const core::dimension2d<u32>& size);


        bool writeImageToFile(IImage* image, const io::path& filename,u32 param);
        bool writeImageToFile(IImage* image, io::IWriteFile * file, u32 param);

    public:

		core::array<video::IImageLoader*> SurfaceLoader;
		core::array<video::IImageWriter*> SurfaceWriter;
        io::IFileSystem *FileSystem;
};

}
}

#endif

